#import "MDMediaPlayback.h"
//#import "MDFFMonitor.h"
#import "MDFFOptions.h"
#import "MDSDLGLViewProtocol.h"

// media meta
#define k_MDM_KEY_FORMAT         @"format"
#define k_MDM_KEY_DURATION_US    @"duration_us"
#define k_MDM_KEY_START_US       @"start_us"
#define k_MDM_KEY_BITRATE        @"bitrate"

// stream meta
#define k_MDM_KEY_TYPE           @"type"
#define k_MDM_VAL_TYPE__VIDEO    @"video"
#define k_MDM_VAL_TYPE__AUDIO    @"audio"
#define k_MDM_VAL_TYPE__UNKNOWN  @"unknown"

#define k_MDM_KEY_CODEC_NAME      @"codec_name"
#define k_MDM_KEY_CODEC_PROFILE   @"codec_profile"
#define k_MDM_KEY_CODEC_LONG_NAME @"codec_long_name"

// stream: video
#define k_MDM_KEY_WIDTH          @"width"
#define k_MDM_KEY_HEIGHT         @"height"
#define k_MDM_KEY_FPS_NUM        @"fps_num"
#define k_MDM_KEY_FPS_DEN        @"fps_den"
#define k_MDM_KEY_TBR_NUM        @"tbr_num"
#define k_MDM_KEY_TBR_DEN        @"tbr_den"
#define k_MDM_KEY_SAR_NUM        @"sar_num"
#define k_MDM_KEY_SAR_DEN        @"sar_den"
// stream: audio
#define k_MDM_KEY_SAMPLE_RATE    @"sample_rate"
#define k_MDM_KEY_CHANNEL_LAYOUT @"channel_layout"

#define k_MDM_KEY_STREAMS       @"streams"

typedef enum MDMediaFFLogLevel {
    k_MD_LOG_UNKNOWN = 0,
    k_MD_LOG_DEFAULT = 1,

    k_MD_LOG_VERBOSE = 2,
    k_MD_LOG_DEBUG   = 3,
    k_MD_LOG_INFO    = 4,
    k_MD_LOG_WARN    = 5,
    k_MD_LOG_ERROR   = 6,
    k_MD_LOG_FATAL   = 7,
    k_MD_LOG_SILENT  = 8,
} MDMediaFFLogLevel;

@class MDFFMonitor;

typedef void(^PTSBlock)(id _Nullable playerController, int value);
typedef void(^FFLogBlock) (const char *fmt, va_list vl, char *line);

@interface MDFFMoviePlayerController : NSObject <MDMediaPlayback>

/**
 options传nil, 内部会默认调用optionsByLive
 */
- (id _Nullable )initWithContentURL:(NSURL *_Nullable)aUrl
                        withOptions:(MDFFOptions *_Nullable)options;

- (id _Nullable)initWithContentURLString:(NSString *_Nullable)aUrlString
                   withOptions:(MDFFOptions *_Nullable)options;

- (id _Nullable)initWithMoreContent:(NSURL *_Nullable)aUrl
             withOptions:(MDFFOptions *_Nullable)options
              withGLView:(UIView<MDSDLGLViewProtocol> *_Nullable)glView;

- (id _Nullable)initWithMoreContentString:(NSString *_Nullable)aUrlString
                 withOptions:(MDFFOptions *_Nullable)options
                  withGLView:(UIView<MDSDLGLViewProtocol> *_Nullable)glView;

- (void)prepareToPlay;
- (void)play;
- (void)pause;
- (void)stop;
- (BOOL)isPlaying;
- (BOOL)isPaused;
- (int64_t)trafficStatistic;
- (float)dropFrameRate;

- (void)setPauseInBackground:(BOOL)pause;
- (BOOL)isVideoToolboxOpen;
- (void)setScalingMode:(MDMPMovieScalingMode)aScalingMode;

- (void)setHudValue:(NSString *)value forKey:(NSString *)key;

+ (void)setLogReport:(BOOL)preferLogReport withBlock:(FFLogBlock)block;
+ (void)setLogLevel:(MDMediaFFLogLevel)logLevel;
+ (BOOL)checkIfFFmpegVersionMatch:(BOOL)showAlert;
+ (BOOL)checkIfPlayerVersionMatch:(BOOL)showAlert
                            version:(NSString *_Nullable)version;

-(NSDictionary *_Nonnull)getState;

@property(nonatomic, readonly) CGFloat fpsInMeta;
@property(nonatomic, readonly) CGFloat fpsAtOutput;
@property(nonatomic) BOOL shouldShowHudView;
@property(nonatomic, copy) NSString *shouldAnalyse;
@property(nonatomic, copy) NSString *analyseUrl;


- (void)setOptionValue:(NSString *)value
                forKey:(NSString *)key
            ofCategory:(MDFFOptionCategory)category;

- (void)setOptionIntValue:(int64_t)value
                   forKey:(NSString *)key
               ofCategory:(MDFFOptionCategory)category;


- (void)setFormatOptionValue:       (NSString *)value forKey:(NSString *)key;
- (void)setCodecOptionValue:        (NSString *)value forKey:(NSString *)key;
- (void)setSwsOptionValue:          (NSString *)value forKey:(NSString *)key;
- (void)setPlayerOptionValue:       (NSString *)value forKey:(NSString *)key;

- (void)setFormatOptionIntValue:    (int64_t)value forKey:(NSString *)key;
- (void)setCodecOptionIntValue:     (int64_t)value forKey:(NSString *)key;
- (void)setSwsOptionIntValue:       (int64_t)value forKey:(NSString *)key;
- (void)setPlayerOptionIntValue:    (int64_t)value forKey:(NSString *)key;


@property (nonatomic, retain) id<MDMediaUrlOpenDelegate> segmentOpenDelegate;
@property (nonatomic, retain) id<MDMediaUrlOpenDelegate> tcpOpenDelegate;
@property (nonatomic, retain) id<MDMediaUrlOpenDelegate> httpOpenDelegate;
@property (nonatomic, retain) id<MDMediaUrlOpenDelegate> liveOpenDelegate;

@property (nonatomic, retain) id<MDMediaNativeInvokeDelegate> _Nullable nativeInvokeDelegate;

- (void)setPTSInterval:(int)interval withBlock:(PTSBlock _Nullable )ptsBlock;

- (void)didShutdown;

#pragma mark KVO properties
@property (nonatomic, readonly) MDFFMonitor *monitor;

@end

#define MD_FF_IO_TYPE_READ (1)
void MDFFIOStatDebugCallback(const char *url, int type, int bytes);
void MDFFIOStatRegister(void (*cb)(const char *url, int type, int bytes));

void MDFFIOStatCompleteDebugCallback(const char *url,
                                      int64_t read_bytes, int64_t total_size,
                                      int64_t elpased_time, int64_t total_duration);
void MDFFIOStatCompleteRegister(void (*cb)(const char *url,
                                            int64_t read_bytes, int64_t total_size,
                                            int64_t elpased_time, int64_t total_duration));
