#import <Foundation/Foundation.h>

typedef enum MDFFOptionCategory {
    kMDFFOptionCategoryFormat = 1,
    kMDFFOptionCategoryCodec  = 2,
    kMDFFOptionCategorySws    = 3,
    kMDFFOptionCategoryPlayer = 4,
    kMDFFOptionCategorySwr    = 5,
} MDFFOptionCategory;

// for codec option 'skip_loop_filter' and 'skip_frame'
typedef enum MDAVDiscard {
    /* We leave some space between them for extensions (drop some
     * keyframes for intra-only or drop just some bidir frames). */
    MD_AVDISCARD_NONE    =-16, ///< discard nothing
    MD_AVDISCARD_DEFAULT =  0, ///< discard useless packets like 0 size packets in avi
    MD_AVDISCARD_NONREF  =  8, ///< discard all non reference
    MD_AVDISCARD_BIDIR   = 16, ///< discard all bidirectional frames
    MD_AVDISCARD_NONKEY  = 32, ///< discard all frames except keyframes
    MD_AVDISCARD_ALL     = 48, ///< discard all
} MDAVDiscard;

struct MdMediaPlayer;

@interface MDFFOptions : NSObject

/**
 常规视频的默认配置
 */
+ (MDFFOptions *)optionsByCommon;

/**
 直播视频的默认配置
 */
+ (MDFFOptions *)optionsByLive;

-(void)applyTo:(struct MdMediaPlayer *)mediaPlayer;

- (void)setOptionValue:(NSString *)value
                forKey:(NSString *)key
            ofCategory:(MDFFOptionCategory)category;

- (void)setOptionIntValue:(int64_t)value
                   forKey:(NSString *)key
               ofCategory:(MDFFOptionCategory)category;


-(void)setFormatOptionValue:       (NSString *)value forKey:(NSString *)key;
-(void)setCodecOptionValue:        (NSString *)value forKey:(NSString *)key;
-(void)setSwsOptionValue:          (NSString *)value forKey:(NSString *)key;
-(void)setPlayerOptionValue:       (NSString *)value forKey:(NSString *)key;

-(void)setFormatOptionIntValue:    (int64_t)value forKey:(NSString *)key;
-(void)setCodecOptionIntValue:     (int64_t)value forKey:(NSString *)key;
-(void)setSwsOptionIntValue:       (int64_t)value forKey:(NSString *)key;
-(void)setPlayerOptionIntValue:    (int64_t)value forKey:(NSString *)key;

@property(nonatomic) BOOL showHudView;
//audioSession初始化，category为playback，options为nil，否则options设置为mix。
@property(nonatomic) BOOL playbackOptionsNil;


@end
