//
//  MDMediaPlayerAdapter.h
//  MDMediaPlayer
//
//  Created by mac-zwj on 2022/3/28.
//  Copyright © 2022 mudu. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <MDPlayerManager/MDPlayerManager.h>

NS_ASSUME_NONNULL_BEGIN

@interface MDMediaPlayerAdapter : NSObject <MDPlayDelegate>

/**
 * 截图回调
 * @note  调用 [snapshot] 截图之后，会收到这个回调通知
 * @param image  已截取的视频画面
 */
@property (nonatomic, copy) void(^onSnapshotComplete)(UIImage* image);

@property (nonatomic, copy) void(^onPlaybackIsPrepared)(void);
@property (nonatomic, copy) void(^onPlaybackDidFinished)(void);
@property (nonatomic, copy) void(^onPlayerEnterBackground)(void);
@property (nonatomic, copy) void(^onPlayerBecomeActive)(void);

/**
 * 直播播放器错误通知，播放器出现错误时，会回调该通知
 * @param code      错误码
 * @param msg       错误信息
 * @param extraInfo 扩展信息
 */
@property (nonatomic, copy) void(^onError)(MDPlayerCode code, NSString* message, NSDictionary* extraInfo);
/**
 * 直播播放器警告通知
 * @param code      警告码
 * @param msg       警告信息
 * @param extraInfo 扩展信息
 */
@property (nonatomic, copy) void(^onWarning)(MDPlayerCode code, NSString* message, NSDictionary* extraInfo);
/**
 * 直播播放器分辨率变化通知
 * @param width     视频宽
 * @param height    视频高
 */
@property (nonatomic, copy) void(^onVideoResolutionChanged)(NSInteger width, NSInteger height);
/**
 * 已经成功连接到服务器
 * @param extraInfo 扩展信息
 */
@property (nonatomic, copy) void(^onConnected)(NSDictionary* extraInfo);
/**
 * 视频播放事件
 * @param firstPlay 第一次播放标志
 * @param extraInfo 扩展信息
 */
@property (nonatomic, copy) void(^onVideoPlaying)(BOOL firstPlay, NSDictionary* extraInfo);
/**
 * 音频播放事件
 * @param firstPlay 第一次播放标志
 * @param extraInfo 扩展信息
 */
@property (nonatomic, copy) void(^onAudioPlaying)(BOOL firstPlay, NSDictionary* extraInfo);
/**
 * 视频加载事件
 * @param extraInfo 扩展信息
 */
@property (nonatomic, copy) void(^onVideoLoading)(NSDictionary* extraInfo);
/**
 * 音频加载事件
 * @param extraInfo 扩展信息
 */
@property (nonatomic, copy) void(^onAudioLoading)(NSDictionary* extraInfo);
/**
 * 播放器音量大小回调
 * @param volume 音量大小
 * @note  调用 [enableVolumeEvaluation] 开启播放音量大小提示之后，会收到这个回调通知。
 */
@property (nonatomic, copy) void(^onPlayoutVolumeUpdate)(NSInteger volume);
/**
 * 直播播放器统计数据回调
 * @param statistics 播放器统计数据
 */
@property (nonatomic, copy) void(^onStatisticsUpdate)(id statistics);
/**
 * 自定义视频渲染回调
 * @param videoFrame 视频帧数据
 * @note  需要您调用 [enableObserveVideoFrame] 开启回调开关
 */
@property (nonatomic, copy) void(^onRenderVideoFrame)(id videoFrame);
/**
 * 收到 SEI 消息的回调，发送端通过 `sendSeiMessage` 来发送 SEI 消息。
 * @note  调用 `enableReceiveSeiMessage` 开启接收 SEI 消息之后，会收到这个回调通知。
 * @param payloadType    回调数据的SEI payloadType
 * @param data     数据
 */
@property (nonatomic, copy) void(^onReceiveSeiMessage)(int payloadType, NSData* data);

+ (instancetype)getAdapter;

@end

NS_ASSUME_NONNULL_END
