//
//  MDPlayerType.h
//  MDPlayerManager
//
//  Created by mac-zwj on 2022/4/7.
//

#ifndef MDPlayerType_h
#define MDPlayerType_h
#endif /* MDPlayerType_h */


typedef NS_ENUM(NSInteger, MDPlayerCode) {

    /// 没有错误
    MDPLAYER_OK = 0,

    /// 暂未归类的通用错误
    MDPLAYER_ERROR_FAILED = -1,

    /// 调用 API 时，传入的参数不合法
    MDPLAYER_ERROR_INVALID_PARAMETER = -2,

    /// API 调用被拒绝
    MDPLAYER_ERROR_REFUSED = -3,

    /// 当前 API 不支持调用
    MDPLAYER_ERROR_NOT_SUPPORTED = -4,

    /// license 不合法，调用失败
    MDPLAYER_ERROR_INVALID_LICENSE = -5,

    /// 请求服务器超时
    MDPLAYER_ERROR_REQUEST_TIMEOUT = -6,

    /// 服务器无法处理您的请求
    MDPLAYER_ERROR_SERVER_PROCESS_FAILED = -7,

    /// 连接断开
    MDPLAYER_ERROR_DISCONNECTED = -8,

    /////////////////////////////////////////////////////////////////////////////////
    //
    //      网络相关的警告码
    //
    /////////////////////////////////////////////////////////////////////////////////

    /// 网络状况不佳：上行带宽太小，上传数据受阻
    MDPLAYER_WARNING_NETWORK_BUSY = 1101,

    /// 当前视频播放出现卡顿
    MDPLAYER_WARNING_VIDEO_BLOCK = 2105,

    /////////////////////////////////////////////////////////////////////////////////
    //
    //             摄像头相关的警告码
    //
    /////////////////////////////////////////////////////////////////////////////////

    /// 摄像头打开失败
    MDPLAYER_WARNING_CAMERA_START_FAILED = -1301,

    /// 摄像头正在被占用中，可尝试打开其他摄像头
    MDPLAYER_WARNING_CAMERA_OCCUPIED = -1316,

    /// 摄像头设备未授权，通常在移动设备出现，可能是权限被用户拒绝了
    MDPLAYER_WARNING_CAMERA_NO_PERMISSION = -1314,

    /////////////////////////////////////////////////////////////////////////////////
    //
    //             麦克风相关的警告码
    //
    /////////////////////////////////////////////////////////////////////////////////

    /// 麦克风打开失败
    MDPLAYER_WARNING_MICROPHONE_START_FAILED = -1302,

    /// 麦克风正在被占用中，例如移动设备正在通话时，打开麦克风会失败
    MDPLAYER_WARNING_MICROPHONE_OCCUPIED = -1319,

    /// 麦克风设备未授权，通常在移动设备出现，可能是权限被用户拒绝了
    MDPLAYER_WARNING_MICROPHONE_NO_PERMISSION = -1317,

    /////////////////////////////////////////////////////////////////////////////////
    //
    //             屏幕分享相关警告码
    //
    /////////////////////////////////////////////////////////////////////////////////

    /// 当前系统不支持屏幕分享
    MDPLAYER_WARNING_SCREEN_CAPTURE_NOT_SUPPORTED = -1309,

    /// 开始录屏失败，如果在移动设备出现，可能是权限被用户拒绝了
    MDPLAYER_WARNING_SCREEN_CAPTURE_START_FAILED = -1308,

    /// 录屏被系统中断
    MDPLAYER_WARNING_SCREEN_CAPTURE_INTERRUPTED = -7001,
    
    ///未初始化Adapter
    MDPLAYER_ERROR_NOADAPTER = 1,
    
    ///未初始化播放器
    MDPLAYER_ERROR_NOPLAYER = 2,
    
    ///adapter未实现函数
    MDPLAYER_ERROR_NOFUNCTION = 3,
    
    ///播放地址为空
    MDPLAYER_ERROR_NOURL = 4,
    
    ///截图错误
    MDPLAYER_ERROR_SNATSHOP = 5,
    
    ///低延时播放器只支持播放RTC流
    MDPLAYER_ERROR_RTCSTREAM_ONLY = 6,
    
    ///MDMediaPlayerAdapter播放视频报错
    MDPLAYER_ERROR_MDMEDIAPLAYER = 7,

};



/**
 * @brief 播放状态
 */
typedef NS_ENUM(NSInteger, MDPlaybackState) {
    /**
     * 播放停止
     */
    MDPlaybackStateStopped = 0,
    /**
     * 开始播放
     */
    MDPlaybackStatePlaying = 1,
    /**
     * 暂停播放
     */
    MDPlaybackStatePaused = 2,
    /**
     * 播放间断
     */
    MDPlaybackStateInterrupted = 3,
    /**
     * 播放快进
     */
    MDPlaybackStateSeekingForward = 4,
    /**
     * 播放后退
     */
    MDPlaybackStateSeekingBackward = 5
};

/**
 * @brief 视频画面顺时针旋转角度。
 */
typedef NS_ENUM(NSInteger, MDPlayerRenderRotation) {
    /**
     * 不旋转
     */
    MDPlayerRenderRotation0 = 0,
    /**
     * 顺时针旋转90度
     */
    MDPlayerRenderRotation90 = 1,
    /**
     * 顺时针旋转180度
     */
    MDPlayerRenderRotation180 = 2,
    /**
     * 顺时针旋转270度
     */
    MDPlayerRenderRotation270 = 3
};

/**
 * @brief 视频画面填充模式。
 */
typedef NS_ENUM(NSInteger, MDPlayerScaleMode) {
    /**
     * 不缩放
     */
    MDPlayerScaleModeNone = 0,
    /**
     * 等比缩放，图像长边填满屏幕，短边区域会被填充黑色，画面的内容完整
     */
    MDPlayerScaleModeAspectFit = 1,
    /**
     * 等比缩放，图像铺满屏幕，超出显示视窗的视频部分将被裁剪，画面显示可能不完整
     */
    MDPlayerScaleModeAspectFill = 2,
    /**
     * 不等比缩放，图像铺满屏幕，图像可能会被拉伸，画面的内容完整
     */
    MDPlayerScaleModeFill = 3
};

/**
 * @brief 视频帧的像素格式。
 */
typedef NS_ENUM(NSInteger, MDPlayerPixelFormat) {
    /**
     * 未知
     */
    MDPlayerPixelFormatUnknown = 0,
    /**
     * YUV420P I420
     */
    MDPlayerPixelFormatI420 = 1,
    /**
     * YUV420SP NV12
     */
    MDPlayerPixelFormatNV12 = 2,
    /**
     * BGRA8888
     */
    MDPlayerPixelFormatBGRA32 = 3,
    /**
     * OpenGL 2D 纹理
     */
    MDPlayerPixelFormatTexture2D = 4
};

/**
 * @brief 视频数据包装格式。
 * @note 在自定义采集和自定义渲染功能，您需要用到下列枚举值来指定您希望以什么样的格式来包装视频数据。
 * - PixelBuffer：直接使用效率最高，iOS 系统提供了众多 API 获取或处理 PixelBuffer
 * - NSData：     当使用自定义渲染时，PixelBuffer拷贝一次到NSData。当使用自定义采集时，NSData拷贝一次到PixelBuffer。因此，性能会受到一定程度的影响
 */
typedef NS_ENUM(NSInteger, MDPlayerBufferType) {
    /**
     * 未知
     */
    MDPlayerBufferTypeUnknown = 0,
    /**
     * 直接使用效率最高，iOS 系统提供了众多 API 获取或处理 PixelBuffer
     */
    MDPlayerBufferTypePixelBuffer = 1,
    /**
     * 会有一定的性能消耗，SDK 内部是直接处理 PixelBuffer 的，所以会存在 NSData 和 PixelBuffer 之间类型转换所产生的内存拷贝开销
     */
    MDPlayerBufferTypeNSData = 2,
    /**
     * 直接操作纹理 ID，性能最好
     */
    MDPlayerBufferTypeTexture = 3
};
