//
//  MDCommenter.h
//  MuduComment
//
//

#import <Foundation/Foundation.h>
#import "MDPlatformUser.h"
#import "MDMessageEntity.h"

typedef void (^MDCommenterBlock) (NSString* data, NSError *err);

typedef enum Environment:NSUInteger{
    Environment_Debug,
    Environment_Uat,
    Environment_Release
}Environment;

@protocol MDMessageDelegate <NSObject>

@required

-(void)onReceiveError:(NSInteger)errorCode withMessage:(NSString *)message;

-(void)onReceiveMessage:(id)commenter withMessage:(NSDictionary *)messageObject withEvent:(NSString *)event;
-(void)onReceiveComment:(id)commenter withComment:(MDMessageEntity *)mesasgeEntity withEvent:(NSString *)event;


@end

@interface MDCommenter : NSObject

@property(nonatomic, weak) id<MDMessageDelegate> messageDelegate;

@property(nonatomic, assign) BOOL isConnect; //是否连接

@property (nonatomic, copy, readonly) NSDictionary *userInfo;

+ (void)setAccountKey:(NSString *)key;

+ (MDCommenter *)instance;

/** 初始化频道actId，即频道hash id */
- (void)initWithActId:(NSString*)actId;

/** 设置环境*/
-(void)setEnvironment:(Environment)environment;

/** 获取用户token*/
- (NSString *)getToken;
/** 获取sessionid*/
- (NSString *)getSessionId;


/** 登录*/
//匿名登录
- (void)login:(NSString*)username avatar:(NSString*)avatarUrl callback:(MDCommenterBlock)callback;
//第三方授权登录
- (void)loginByOther:(MDPlatformUser *)user callback:(MDCommenterBlock)callback;
//已有token登录
- (void)loginByToken:(NSString *)token callback:(MDCommenterBlock)callback;

/** 发送数据 */
- (void)publishMessage:(NSString*)message callback:(MDCommenterBlock)callback;

/** 重连 */
- (void)reconnect;

/** 页面退出*/
- (void)close;

/** 当前版本*/
+ (NSString *)version;







#pragma mark subscribe other topics
/** 开始订阅 */
//- (void)subscribe:(MDStateChangeBlock)stateChangeBlock;

/**
 添加订阅 - 点赞
 */
- (void)addSubscribeLike;

/**
 添加订阅 - 打赏
 */
- (void)addSubscribeDonate;

/**
 添加订阅 - 文档
 */
- (void)addSubscribeDoc;

/**
 添加订阅 - commodity
 */
- (void)addSubscribeCommodity;

/**
 添加订阅 - 投票
 */
- (void)addSubscribeVote:(NSString *)actInfoId;

/**
 添加订阅 - 问卷
 */
- (void)addSubscribeQuestionaire;

/**
 添加订阅 - 红包
 */
- (void)addSubscribeInitRedPack:(NSString *)actInfoId redpackHashId:(NSString*)redpackHashId;

/**
 添加订阅 - 红包
 */
- (void)addSubscribeCashRedPack:(NSString *)actInfoId redpackHashId:(NSString*)redpackHashId;

/**
 添加订阅 - 红包
 */
- (void)addSubscribeAwardRedPack:(NSString *)actInfoId redpackHashId:(NSString*)redpackHashId;

/**
 添加订阅 - 菜单
 */
- (void)addSubscribeMenu;

/**
 添加订阅 - 流
 */
- (void)addSubscribeStream:(NSString *)streamId;

@end
