//
//  MDLogManager.h
//  MuduLibrary
//
//  Created by mac-zwj on 2021/12/6.
//

#import <Foundation/Foundation.h>

NS_ASSUME_NONNULL_BEGIN

@interface MDLogManager : NSObject


typedef NSString * _Nonnull MDLogSource NS_STRING_ENUM;

/**企播连麦*/
FOUNDATION_EXPORT MDLogSource const _Nonnull MDLogSourceBuguRTCSDK;
/**企播新连麦*/
FOUNDATION_EXPORT MDLogSource const _Nonnull MDLogSourceBuguMRTCSDK;
/**企播评论*/
FOUNDATION_EXPORT MDLogSource const _Nonnull MDLogSourceBuguCommentSDK;
/**企播服务*/
FOUNDATION_EXPORT MDLogSource const _Nonnull MDLogSourceBuguServiceSDK;

/**推流*/
FOUNDATION_EXPORT MDLogSource const _Nonnull MDLogSourceMuduPushSDK;
/**拉流*/
FOUNDATION_EXPORT MDLogSource const _Nonnull MDLogSourceMuduPlayerSDK;
/**直播评论*/
FOUNDATION_EXPORT MDLogSource const _Nonnull MDLogSourceMuduCommentSDK;
/**白板*/
FOUNDATION_EXPORT MDLogSource const _Nonnull MDLogSourceMuduWhiteboardSDK;
/**直播*/
FOUNDATION_EXPORT MDLogSource const _Nonnull MDLogSourceMudutvPageSDK;

/**异常*/
FOUNDATION_EXPORT MDLogSource const _Nonnull MDLogSourceException;
/**其他*/
FOUNDATION_EXPORT MDLogSource const _Nonnull MDLogSourceOther;


/**
 日志级别
 */
typedef NS_OPTIONS(NSUInteger, MDLogLevel) {
    MDLogLevel_None          = 0,            // 无日志
    MDLogLevel_Error         = 1 << 0,       // 错误日志
    MDLogLevel_Warning       = 1 << 1,       // 警告日志
    MDLogLevel_Info          = 1 << 2,       // 信息日志
    MDLogLevel_Debug         = 1 << 3,       // 调试日志
    // 全部日志
    MDLogLevel_All           = MDLogLevel_Error | MDLogLevel_Warning | MDLogLevel_Info | MDLogLevel_Debug,
};


/**
 即将废弃，暂时保留防止编译报错
 */
#define MDLog(format, ...)  \
MDLibraryLog(MDLogSourceOther, MDLogLevel_Debug, format, ##__VA_ARGS__);


/**
 打印并缓存
 */
#define MDLibraryLog(source, logLevel, format, ...) \
[MDLogManager mdLogPrintSaveWithSource: \
source   \
level:logLevel  \
file:__FILE_NAME__  \
func:__FUNCTION__   \
line:__LINE__   \
fmt:[NSString stringWithFormat:format, ##__VA_ARGS__]];

/**
 打印
 */
#define MDLibraryLogPrint(source, logLevel, format, ...)  \
[MDLogManager mdLogPrintWithSource: \
source   \
level:logLevel  \
file:__FILE_NAME__  \
func:__FUNCTION__   \
line:__LINE__   \
fmt:[NSString stringWithFormat:format, ##__VA_ARGS__]];

/**
 缓存
 */
#define MDLibraryLogSave(source, logLevel, format, ...)  \
[MDLogManager mdLogSaveWithSource:  \
source    \
level:logLevel  \
file:__FILE_NAME__  \
func:__FUNCTION__   \
line:__LINE__   \
fmt:[NSString stringWithFormat:format, ##__VA_ARGS__]];

/**
 打印并缓存
 */
#define MDLibraryDetailLog(source, logLevel, fileName, funcName, lineNum, str) \
[MDLogManager mdLogPrintSaveWithSource: \
source   \
level:logLevel  \
file:__FILE_NAME__  \
func:__FUNCTION__   \
line:__LINE__   \
fmt:str];

/**
 打印并缓存
 */
+ (void)mdLogPrintSaveWithSource:(MDLogSource)source
                           level:(MDLogLevel)logLevel
                            file:(const char *)file
                            func:(const char *)function
                            line:(int)line
                             fmt:(NSString *)format;

/**
 打印
 */
+ (void)mdLogPrintWithSource:(MDLogSource)source
                       level:(MDLogLevel)logLevel
                        file:(const char *)file
                        func:(const char *)function
                        line:(int)line
                         fmt:(NSString *)format;

/**
 缓存
 */
+ (void)mdLogSaveWithSource:(MDLogSource)source
                      level:(MDLogLevel)logLevel
                       file:(const char *)file
                       func:(const char *)function
                       line:(int)line
                        fmt:(NSString *)format;



/**
 设置日志类型，默认MDLogLevel_None
 */
+ (void)setLogLevel:(MDLogLevel)logLevel;

/**
 获取日志类型
 */
+ (MDLogLevel)getLogLevel;

/**
 设置日志输出开关，默认关闭
 */
+ (void)setLogEnable:(BOOL)enable;

/**
 获取日志输出开关
 */
+ (BOOL)getLogEnable;

/**
 设置日志缓存开关，默认打开
 */
+ (void)setLogCachesEnable:(BOOL)enable;

/**
 获取日志缓存开关
 */
+ (BOOL)getLogCachesEnable;

/**
 硬盘存储天数，默认10
 */
+ (void)setDiskCacheCount:(int)diskCacheCount;

/**
 单个文件最大存储量，默认1024*1024
 */
+ (void)setMaxFileSize:(NSUInteger)maxFileSize;

/**
 清除日志缓存
 */
+ (void)clearLogCaches;



@end

NS_ASSUME_NONNULL_END
