//
//  MDTool.h
//  MuduLibrary
//
//  Created by mac-zwj on 2022/1/14.
//

#import <Foundation/Foundation.h>
#import <UIKit/UIKit.h>

NS_ASSUME_NONNULL_BEGIN

@interface MDTool : NSObject

/**
 获取SDK版本号
 */
+ (NSString *)version;

/**
 Null / Nil / "null" / "NULL"
 */
+ (BOOL)isNullOrNil:(id)obj;

/**
 RRGGBBAA
 */
+ (UIColor *)colorWithHex:(NSInteger)hex;

/**
 RRGGBB
 */
+ (UIColor*)colorWithHex:(NSInteger)hex alpha:(CGFloat)alpha;

/**
 #RGB / #RGBA / #RRGGBB / #RRGGBBAA
 */
+ (UIColor *)colorWithHexString:(NSString *)hexString;

/**
 #RGB / #RGBA / #RRGGBB / #RRGGBBAA
 */
+ (UIColor *)colorWithHexString:(NSString *)hexString alpha:(CGFloat)alpha;

/**
 UIColor转#FFFFFF格式的字符串
 */
+ (NSString *)hexStringFromColor:(UIColor *)color;

/**
 卸载应用重新安装后会不一致
 */
+ (NSString *)getUUID;

/**
 Dictionary转Json字符串，并去除空格和换行
 */
+ (NSString *)jsonStringWithDictionary:(NSDictionary *)dict;

/**
 json字符串转Dictionary
 */
+ (NSDictionary *)dictionaryWithJsonString:(NSString *)jsonString;

/**
 *  32位md5加密算法
 *  @param str 传入要加密的字符串
 *  @return NSString
 */
+ (NSString *)md5:(NSString *)str;

/**
 产生随机字符串
 */
+ (NSString *)randomStrByLength:(int)length;

/**
 字符串转时间
 */
+ (NSDate *)convertDateString:(NSString *)str;

/**
 根据日期算出周几
 */
+ (NSString*)weekdayStringFromDate:(NSDate*)inputDate;

/**
*  url decode 解码
*/
+ (NSString *)URLDecodedString:(NSString *)str;

/**
*  url encode 编码
*/
+ (NSString *)URLEncodedString:(NSString *)str;

/**
 将 &lt 等类似的字符转化为HTML中的“<”等
 */
+ (NSString *)htmlEntityDecode:(NSString *)string;

/**
 验证手机号
 */
+(BOOL)isVerifyPhoneNumber:(NSString *)phoneNumber;

/**
 身份证号验证
 */
+ (BOOL)isIdentityCard:(NSString *)identityCard;

/**
 邮箱验证
 */
+ (BOOL)isEmail:(NSString *)email;

/** 银行卡号有效性问题Luhn算法
 *  现行 16 位银联卡现行卡号开头 6 位是 622126～622925 之间的，7 到 15 位是银行自定义的，
 *  可能是发卡分行，发卡网点，发卡序号，第 16 位是校验码。
 *  16 位卡号校验位采用 Luhm 校验方法计算：
 *  1，将未带校验位的 15 位卡号从右依次编号 1 到 15，位于奇数位号上的数字乘以 2
 *  2，将奇位乘积的个十位全部相加，再加上所有偶数位上的数字
 *  3，将加法和加上校验位能被 10 整除。
 */
+ (BOOL)isBankCardluhmCheck:(NSString *)bankNumber;

/**
 获取statusbar 高度
 */
+(CGFloat)getStatusBarHeight;

/**
 获取statusbar和navigation bar高度
 */
+(CGFloat)getStatusBarAndNavigationBarHeight;

/**
 获取底部安全区域高度
 */
+(CGFloat)getSafeAreaInsetsBottomHeight;

/**
 获取顶部部安全区域高度
 */
+(CGFloat)getSafeAreaInsetsTopHeight;

/**
 通过颜色获取图片
 */
+ (UIImage *)createImageWithColor:(UIColor *)color size:(CGSize)size;

/**
 给图片裁剪圆角
 */
+ (UIImage *)setCornerWithImage:(UIImage *)image cornerRadius:(CGFloat)cornerRadius;

/**
 修改图片尺寸
 */
+ (UIImage *)setSizeWithImage:(UIImage *)image toSize:(CGSize)size;

/**
 压缩图片质量
 */
+ (UIImage *)compressImageQuality:(UIImage *)image toByte:(NSInteger)maxLength;

/**
 获取iOS13 前/后 的keyWindow
 */
+ (UIWindow *)keyWindow;

/**
 沙盒目录：document
 */
+ (NSString*)documentPath;

/**
 创建目录
 */
+ (BOOL)createDirectory:(NSString *)directioryPath;

/**
 创建新文件
 */
+ (BOOL)createFile:(NSString *)filePath;

/**
 App 内存占用量
 */
+ (int64_t)memoryUsage;

/**
 App CPU占用率
 */
+ (double)cpuUsage;

@end

NS_ASSUME_NONNULL_END
