//
//  ViewController.m
//  mudutv_ios_player
//
//  Created by mudu on 2017/6/1.
//  Copyright © 2017年 MuduTV. All rights reserved.
//

#import <MuduComment/MuduComment.h>
#import "ViewController.h"
#import "SVProgressHUD.h"
#import <MuduLibrary/MuduLibrary.h>
#import <MDPlayerManager/MDPlayerManager.h>
#import <MDMediaFrameworkWithSSL/MDMediaPlayerAdapter.h>


#import <MediaPlayer/MPMediaItem.h>
#import <MediaPlayer/MPNowPlayingInfoCenter.h>

//#import <MediaPlayer/MPRemoteCommandCenter.h>

@interface ViewController () <UITableViewDelegate, UITableViewDataSource, UISearchBarDelegate, MDMessageDelegate>

@property (nonatomic, strong) NSMutableArray* urls;
@property (nonatomic, assign) NSInteger urlIndex;
@property (nonatomic, strong) NSMutableArray* playerFunctions;
@property (nonatomic, strong) UIImage* snapshopImg;
@property (nonatomic, strong) NSTimer* processTimer;
@property (nonatomic, copy) NSString *actId;
@property (nonatomic, copy) NSString *nickname;
@property (nonatomic, copy) NSString *mSourceURL;

@property (nonatomic, strong) NSDictionary *userInfo;

@property (nonatomic, assign) BOOL mPlaying;

@property (nonatomic, strong) UISlider* slider;
@property (nonatomic, strong) UILabel* playbackTimeLbl;
@property (nonatomic, strong) UITableView* playerTableView;
@property (nonatomic, strong) MDPlayerLiveView* liveView;

@property (nonatomic, strong) UITableView *commentTableView;
@property (nonatomic, strong) UITextField *messageField;
@property (nonatomic, strong) UIButton *sendButton;
@property (nonatomic, strong) NSMutableArray *messageArray;

@end

@implementation ViewController

- (void)viewDidLoad {
    [super viewDidLoad];

    self.automaticallyAdjustsScrollViewInsets = YES;

    [self initData];
    [self initSubviews];
    [self initLiveView];
    [self addObservers];
    
    [self initComment];
    
    [self startProcessTimer];
    [self startPlay];

    // 阻止锁屏
    [self.view setBackgroundColor:[UIColor whiteColor]];

    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(keyboardWillChangeFrame:) name:UIKeyboardWillChangeFrameNotification object:nil];
    
    [MDLogManager setLogEnable:NO];
    [MDLogManager setLogLevel:MDLogLevel_None];
}


- (void)dealloc {
    [self.liveView destory];
    [self stopProcessTimer];
    [[NSNotificationCenter defaultCenter] removeObserver:self];
    
    [[MDCommenter instance] close];
}

- (void)initData {
    self.urls = [@[
        @"http://vfx.mtime.cn/Video/2019/03/19/mp4/190319222227698228.mp4",
        @"https://vfx.mtime.cn/Video/2016/11/21/mp4/161121065305521110.mp4",
        @"https://imss-video.huawei.com/video/play/8a821e176554d8bb01658618597b0468/8a821e1768f77742016a59d5a6bf1f8c/28.m3u8",
        @"http://imss-video.huawei.com/video/play/8a821e147825bab401798ae6796a52ba/e5de4088b5704821a44b1f0b9f0cd8d9/22.m3u8"
    ] mutableCopy];
    if (self.mSourceURL) {
        [self.urls insertObject:self.mSourceURL atIndex:0];
    }
    self.urlIndex = 0;
    self.playerFunctions = [@[
        @"切换播放地址",
        @"切换画面填充模式",
        @"音量+",
        @"音量-",
        @"播放",
        @"暂停",
        @"截图",
        @"播放器状态信息的调试浮层",
        @"后台暂停设置",
        @"播放速度+",
        @"播放速度-",
        @"后台暂停设置",
        @"可播放时长"
    ] mutableCopy];
    
    self.messageArray = [[NSMutableArray alloc] init];
}

- (void)initSubviews {

    CGRect rect = self.view.bounds;
    
    int y = 100;
    
    self.slider = [[UISlider alloc] initWithFrame:CGRectMake(20, y, rect.size.width - 60, 40)];
    [self.slider addTarget:self action:@selector(didSliderTouchDown:) forControlEvents:UIControlEventTouchDown];
    [self.slider addTarget:self action:@selector(didSliderTouchCancel:) forControlEvents:UIControlEventTouchCancel];
    [self.slider addTarget:self action:@selector(didSliderTouchUpOutside:) forControlEvents:UIControlEventTouchUpOutside];
    [self.slider addTarget:self action:@selector(didSliderTouchUpInside:) forControlEvents:UIControlEventTouchUpInside];
    [self.slider addTarget:self action:@selector(didSliderValueChanged:) forControlEvents:UIControlEventValueChanged];
    [self.view addSubview:self.slider];
    

    self.playbackTimeLbl = [[UILabel alloc] initWithFrame:CGRectMake(rect.size.width - 40, y, rect.size.width - 80, 40)];
    self.playbackTimeLbl.textColor = UIColor.redColor;
    self.playbackTimeLbl.font = [UIFont systemFontOfSize:11];
    [self.view addSubview:self.playbackTimeLbl];
    
    
    y+=40;

    self.playerTableView = [[UITableView alloc] initWithFrame:CGRectMake(0, y, rect.size.width, 300) style:UITableViewStylePlain];
    self.playerTableView.delegate = self;
    self.playerTableView.dataSource = self;
    self.playerTableView.backgroundColor = UIColor.clearColor;
    self.playerTableView.rowHeight = 40;
    self.playerTableView.tableFooterView = UIView.new;
    [self.playerTableView registerClass:UITableViewCell.class forCellReuseIdentifier:NSStringFromClass(UITableViewCell.class)];
    [self.view addSubview:self.playerTableView];
    
    y+=300;
    
    UILabel *commentLabel = [[UILabel alloc] initWithFrame:CGRectMake(20, y, rect.size.width-40, 40)];
    commentLabel.textColor = UIColor.blackColor;
    commentLabel.font = [UIFont boldSystemFontOfSize:15];
    commentLabel.text = @"评论列表消息";
    [self.view addSubview:commentLabel];
    
    y+=40;
    
    self.commentTableView = [[UITableView alloc] initWithFrame:CGRectMake(0, y, rect.size.width, rect.size.height-350) style:UITableViewStylePlain];
    self.commentTableView.dataSource = self;
    self.commentTableView.delegate = self;
    self.commentTableView.backgroundColor = UIColor.whiteColor;
    [self.commentTableView setKeyboardDismissMode:UIScrollViewKeyboardDismissModeOnDrag];
    [self.view addSubview:self.commentTableView];
    
    self.messageField = [[UITextField alloc] initWithFrame:CGRectMake(0, rect.size.height-50, rect.size.width-80, 50)];
    [self.messageField setBackgroundColor:[UIColor grayColor]];
    [self.view addSubview:self.messageField];
    self.sendButton = [[UIButton alloc] initWithFrame:CGRectMake(rect.size.width-80, rect.size.height-50, 80, 50)];
    [self.sendButton addTarget:self action:@selector(sendButtonClick) forControlEvents:UIControlEventTouchUpInside];
    [self.sendButton setTitle:@"发送" forState:UIControlStateNormal];
    [self.sendButton setBackgroundColor:[UIColor blueColor]];
    [self.view addSubview:self.sendButton];
}

//##### 评论

-(void)initComment
{
    [MDCommenter setAccountKey:@""];
    [[MDCommenter instance] initWithActId:self.actId];
    [[MDCommenter instance] login:self.nickname avatar:@"http://mudu.tv/assets/img/activity/pc/logo.png" callback:^(NSString *data, NSError *err) {
         if (err == nil) {
             
             self.userInfo = [MDCommenter instance].userInfo;
             NSLog(@"%@",[MDCommenter instance].userInfo);
             
             NSLog(@"%@", [[MDCommenter instance] getToken]);
             
             
             [self.messageArray addObject:[NSString stringWithFormat:@"登录成功:%@ %@",
                                           [MDCommenter instance].userInfo[@"user"][@"nick"],
                                           [[MDCommenter instance] getToken]]];
         }else{
             NSLog(@"%@", err);
             [self.messageArray addObject:[NSString stringWithFormat:@"登录失败:%@", err.localizedDescription]];
         }
         [self.commentTableView reloadData];
    }];
    
    [[MDCommenter instance] setMessageDelegate:self];
    
}


#pragma mark MDMessageDelegate Start

-(void)onReceiveError:(NSInteger)errorCode withMessage:(NSString *)message
{
    
    NSLog(@"receive errorcode = %ld, message = %@", (long)errorCode, message);
    
}


-(void)onReceiveMessage:(id)commenter withMessage:(NSDictionary *)message withEvent:(NSString *)event
{
    //除评论消息外的其他消息
    NSLog(@"receive message = %@ event=%@", message, event);
    
    
    [self.messageArray addObject:[NSString stringWithFormat:@"指令：%@ event：%@", message[@"type"], event]];
    [self.commentTableView reloadData];
    
}
-(void)onReceiveComment:(id)commenter withComment:(MDMessageEntity *)mesasgeEntity withEvent:(NSString *)event
{
    //评论消息
    NSLog(@"receive avatar = %@", mesasgeEntity.content.avatar);
    NSLog(@"receive username = %@", mesasgeEntity.content.username);
    NSLog(@"receive comment = %@", mesasgeEntity.content.message);
    if(![mesasgeEntity.content.userHashId isEqualToString:[[self.userInfo objectForKey:@"user"] objectForKey:@"hash_id"]]){
        
        NSString *chatMessage = mesasgeEntity.content.message;
        
        NSString *emojiRegularExpression = @"<img.*?>";
        NSRegularExpression *emojiRegular = [[NSRegularExpression alloc] initWithPattern:emojiRegularExpression options:NSRegularExpressionDotMatchesLineSeparators error:nil];
        NSString * message = [emojiRegular stringByReplacingMatchesInString:chatMessage options:NSMatchingWithTransparentBounds range:NSMakeRange(0, chatMessage.length)  withTemplate:@""];
        [self.messageArray addObject:message];
        [self.commentTableView reloadData];
        [self.commentTableView scrollToRowAtIndexPath:[NSIndexPath indexPathForRow:self.messageArray.count-1 inSection:0] atScrollPosition:UITableViewScrollPositionBottom animated:YES];
        
    }
}
#pragma mark MDMessageDelegate End




- (void)initLiveView {
    self.liveView = [MDPlayerLiveView liveView];
    [self.liveView setAdapter:[MDMediaPlayerAdapter getAdapter]];
    [self.liveView setFrame:CGRectMake(0, 120, self.view.bounds.size.width, self.view.bounds.size.width * 9 / 16)];
    [self.view insertSubview:self.liveView atIndex:0];
}

- (void)addObservers {
    __weak typeof(self) selfWeak = self;

    //直播播放器错误通知，播放器出现错误时，会回调该通知
    self.liveView.onError = ^(MDPlayerCode code, NSString * _Nonnull message, NSDictionary * _Nonnull extraInfo) {
        NSLog(@"错误信息 - %lu - %@ - %@", code, message, extraInfo);
        if (code == MDPLAYER_ERROR_DISCONNECTED) {
            [SVProgressHUD showErrorWithStatus:@"直播已结束，开始播放下一个视频"];
        } else {
            [SVProgressHUD showErrorWithStatus:[NSString stringWithFormat:@"播放失败 - %li，开始播放下一个视频", code]];
            
        }
        //列表播放
        [selfWeak changePlayUrl];
    };

    //直播播放器警告通知
    self.liveView.onWarning = ^(MDPlayerCode code, NSString * _Nonnull message, NSDictionary * _Nonnull extraInfo) {
        NSLog(@"警告信息 - %lu - %@ - %@", code, message, extraInfo);
    };

    //截图
    self.liveView.onSnapshotComplete = ^(UIImage * _Nonnull image) {
        selfWeak.snapshopImg = image;
        CGFloat width = CGRectGetWidth(selfWeak.view.frame) / 2.f;
        CGFloat height = width * image.size.height / image.size.width;
        CGFloat originX = CGRectGetWidth(selfWeak.view.frame) - width;
        CGFloat originY = CGRectGetHeight(selfWeak.view.frame) - height - 20;
        if (@available(iOS 11.0, *)) {
            UIEdgeInsets inserts = selfWeak.view.safeAreaInsets;
            originY = originY - inserts.bottom;
        } else {
            // Fallback on earlier versions
        }
        UIImageView* imageView = [[UIImageView alloc] initWithFrame:CGRectMake(originX, originY, width, height)];
        imageView.image = image;
        imageView.contentMode = UIViewContentModeScaleAspectFit;
        [selfWeak.view addSubview:imageView];
        [imageView performSelector:@selector(removeFromSuperview) withObject:nil afterDelay:5 inModes:@[NSRunLoopCommonModes]];
    };

    //开始播放
    self.liveView.onPlaybackIsPrepared = ^{
        NSLog(@"播放开始");
        [SVProgressHUD showSuccessWithStatus:@"播放开始"];
        selfWeak.slider.maximumValue = selfWeak.liveView.duration;
        selfWeak.slider.minimumValue = 0;
    };

    //播放结束
    self.liveView.onPlaybackDidFinished = ^{
        NSLog(@"播放结束");
        [SVProgressHUD showSuccessWithStatus:@"播放结束，开始播放下一个视频"];
        //列表播放
        [selfWeak changePlayUrl];
    };

    //进入后台
    self.liveView.onPlayerEnterBackground = ^{
        NSLog(@"进入后台");
    };

    //进入前台
    self.liveView.onPlayerBecomeActive = ^{
        NSLog(@"进入前台");
    };
}



- (void)keyboardWillChangeFrame:(NSNotification *)noti
{
    CGFloat duration = [noti.userInfo[@"UIKeyboardAnimationDurationUserInfoKey"] floatValue];
    CGRect frame = [noti.userInfo[@"UIKeyboardFrameEndUserInfoKey"] CGRectValue];
    CGFloat offsetY = frame.origin.y -   self.view.frame.size.height;
    
    [UIView animateWithDuration:duration animations:^{
        self.view.transform = CGAffineTransformMakeTranslation(0, offsetY);
    }];
}

- (void)sendButtonClick {
    NSString *message = self.messageField.text;

    [[MDCommenter instance] publishMessage:message callback:^(NSString *data, NSError *err) {
        if(err == nil) {
            NSLog(@"发送成功");
            [self.messageArray addObject:message];
        } else {
            //发送失败
            NSLog(@"%@", data);
            [self.messageArray addObject:[NSString stringWithFormat:@"发送失败：%@", err.localizedDescription]];

        }
        
        [self.commentTableView reloadData];
    }];
    [self.messageField setText:@""];
    [self.messageField resignFirstResponder];
}

-(void)configActivityId:(NSString *)activityId liveUrl:(NSString *)playLiveUrl nickname:(NSString *)nickname
{
    self.actId = activityId;
    self.mSourceURL = playLiveUrl;
    self.nickname = nickname;
}

- (void)changePlayUrl {
    self.urlIndex++;
    if (self.urlIndex > self.urls.count-1) {
        self.urlIndex = 0;
    }
    [self startPlay];
    [self.playerTableView reloadData];
}

- (void)changeScaleMode {
    if ([self.liveView getScaleMode] == MDPlayerScaleModeFill) {
        [self.liveView setScaleMode:MDPlayerScaleModeNone];
    } else {
        [self.liveView setScaleMode:[self.liveView getScaleMode]+1];
    }
    [self.playerTableView reloadData];
}

- (void)startPlay {
    MDPlayerCode code = [self.liveView startPlay:self.urls[self.urlIndex]];
    if (code != MDPLAYER_OK) {
        [SVProgressHUD showErrorWithStatus:[NSString stringWithFormat:@"播放失败，code = %li", code]];
    }
}

- (void)stopPlay {
    [self.liveView stopPlay];
}

- (void)pausePlay {
    [self.liveView pausePlay];
}

- (void)resumePlay {
    [self.liveView resumePlay];
}

#pragma mark - UITableViewDelegate

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    if (tableView == self.playerTableView) {
        return self.playerFunctions.count;
    } else {
        return self.messageArray.count;
    }
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    NSString* ID = NSStringFromClass(UITableViewCell.class);
    UITableViewCell* cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleSubtitle reuseIdentifier:ID];
    if (!cell) {
        cell = [tableView dequeueReusableCellWithIdentifier:NSStringFromClass(UITableViewCell.class)];
    }
    cell.backgroundColor = UIColor.clearColor;
    cell.textLabel.textColor = UIColor.redColor;
    cell.textLabel.textAlignment = NSTextAlignmentCenter;
    cell.textLabel.text = nil;
    cell.detailTextLabel.textColor = UIColor.greenColor;
    cell.detailTextLabel.text = nil;
    cell.imageView.image = nil;

    if (tableView == self.commentTableView) {
        cell.textLabel.text = [self.messageArray objectAtIndex:(long)indexPath.row];
        return cell;
    }
    
    cell.textLabel.text = self.playerFunctions[indexPath.row];
    if (indexPath.row == 0) {
        cell.detailTextLabel.text = [NSString stringWithFormat:@"%li", self.urlIndex];
    } else if (indexPath.row == 1) {
        cell.detailTextLabel.text = [NSString stringWithFormat:@"%lu", [self.liveView getScaleMode]];
    } else if (indexPath.row == 2) {
        cell.detailTextLabel.text = [NSString stringWithFormat:@"%lu", [self.liveView getVolume]];
    } else if (indexPath.row == 3) {
        cell.detailTextLabel.text = [NSString stringWithFormat:@"%lu", [self.liveView getVolume]];
    } else if (indexPath.row == 4) {
        if (self.liveView.isPlaying || self.liveView.isPaused) {
            cell.textLabel.text = @"停止";
        } else {
            cell.textLabel.text = @"播放";
        }
    } else if (indexPath.row == 5) {
        if (self.liveView.isPaused) {
            cell.textLabel.text = @"继续";
        } else {
            cell.textLabel.text = @"暂停";
        }
    } else if (indexPath.row == 6) {
        cell.imageView.image = self.snapshopImg;
    } else if (indexPath.row == 7) {
        cell.detailTextLabel.text = self.liveView.isShowDebugView ? @"YES" : @"NO";
    } else if (indexPath.row == 8) {
        cell.detailTextLabel.text = self.liveView.pauseInBackground ? @"YES" : @"NO";
    } else if (indexPath.row == 9) {
        cell.detailTextLabel.text = [NSString stringWithFormat:@"%f", self.liveView.playbackRate];
    } else if (indexPath.row == 10) {
        cell.detailTextLabel.text = [NSString stringWithFormat:@"%f", self.liveView.playbackRate];
    } else if (indexPath.row == 11) {
        cell.detailTextLabel.text = self.liveView.pauseInBackground ? @"YES" : @"NO";
    } else if (indexPath.row == 12) {
        cell.detailTextLabel.text = [NSString stringWithFormat:@"%f", self.liveView.playableDuration];
    }

    return cell;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    [tableView deselectRowAtIndexPath:indexPath animated:YES];
    if (tableView == self.commentTableView) {
        return;
    }
    
    switch (indexPath.row) {
        case 0: {
            [self changePlayUrl];
            break;
        }
        case 1: {
            [self changeScaleMode];
            break;
        }
        case 2: {
            [self.liveView setVolume:[self.liveView getVolume]+10];
            break;
        }
        case 3: {
            [self.liveView setVolume:[self.liveView getVolume]-10];
            break;
        }
        case 4: {
            if (self.liveView.isPlaying || self.liveView.isPaused) {
                [self stopPlay];
            } else {
                [self startPlay];
            }
            break;
        }
        case 5: {
            if (self.liveView.isPaused) {
                [self resumePlay];
            } else {
                [self pausePlay];
            }
            break;
        }
        case 6: {
            [self.liveView snapshot];
            break;
        }
        case 7: {
            self.liveView.isShowDebugView = !self.liveView.isShowDebugView;
            break;
        }
        case 8: {
            self.liveView.pauseInBackground = !self.liveView.pauseInBackground;
            break;
        }
        case 9: {
            self.liveView.playbackRate += 0.1;
            break;
        }
        case 10: {
            self.liveView.playbackRate -= 0.1;
            break;
        }
        case 11: {
            self.liveView.pauseInBackground = !self.liveView.pauseInBackground;
            break;
        }
        case 12: {
            
            break;
        }
        default:
            break;
    }

    [self.playerTableView reloadData];
    [self.view endEditing:YES];
}

- (void)scrollViewDidScroll:(UIScrollView *)scrollView {
    [self.view endEditing:YES];
}


#pragma mark - Slider Action

- (void)didSliderTouchDown:(UISlider *)sender {
    NSLog(@"%s", __func__);
}

- (void)didSliderTouchCancel:(UISlider *)sender {
    NSLog(@"%s", __func__);
}

- (void)didSliderTouchUpOutside:(UISlider *)sender {
    NSLog(@"%s", __func__);
}

- (void)didSliderTouchUpInside:(UISlider *)sender {
    NSLog(@"%s", __func__);
    self.liveView.currentPlaybackTime = _slider.value;
}

- (void)didSliderValueChanged:(UISlider *)sender {
    NSLog(@"%s", __func__);
}

#pragma mark - sliderTimer

- (void)startProcessTimer {
    [self stopProcessTimer];
    __weak typeof(self) selfWeak = self;
    self.processTimer = [NSTimer timerWithTimeInterval:0.5 repeats:YES block:^(NSTimer * _Nonnull timer) {
        if (!selfWeak.slider.isTouchInside) {
            CGFloat currentPlaybackTime = selfWeak.liveView.currentPlaybackTime;
            CGFloat duration = selfWeak.liveView.duration;
            selfWeak.slider.value = currentPlaybackTime;
            selfWeak.playbackTimeLbl.text = [NSString stringWithFormat:@"%d/%d", (int)currentPlaybackTime, (int)duration];
        }
        [selfWeak.playerTableView reloadData];
    }];
    [[NSRunLoop mainRunLoop] addTimer:self.processTimer forMode:NSRunLoopCommonModes];
}


- (BOOL)canBecomeFirstResponder
{
    return YES;
}
- (void)stopProcessTimer {
    if (self.processTimer) {
        [self.processTimer invalidate];
        self.processTimer = nil;
    }
}

@end
